## How to Contribute

I try to maintain my libraries to meet the needs of all their users - so, to that extent, most contributions will be accepted so long as they represent some kind of functional improvement. I'd much prefer to work together and resolve an issue than turn any genuine effort away. To that end, **if you need help with this process, have any questions or confusion, or want to get feedback before a contribution, please don't hesitate to get in touch.** (either [discord](https://discord.jfenn.me) or [email](mailto:dev@jfenn.me) work fine)

This repository has two persistent branches: `main` and `develop` - of the two, most pull requests should be made to the latter. `main` will always contain the source code of the current stable release, so any new changes should be merged into `develop` first. The exception to this is any changes to metadata: the README, documentation, code of conduct, etc. - since these don't affect the compiled program, it makes sense to merge these into `main` immediately, unless they are tied to a change in functionality (changes in a new version of the library, for example).

### Example contributions

- *Development:* Developers can help Alarmio by fixing bugs, implementing features, or helping to debug & research new issues. I'm hoping to write a complete guide to this process in the future - for now, please refer to [CONTRIBUTING.md](./.github/CONTRIBUTING.md).
- *Design:* Attribouter should be intuitive and accessible to a wide variety of users - suggestions to improve certain interfaces are always welcome. This includes compatibility with screen readers, problems with contrast / color blindness, and the sizing/positioning of touch targets in the UI - many of which are shamefully untested in its present state.
- *Localization:* If Attribouter doesn't have support for your fluent language(s), please consider translating it! Most in-app text is stored in [strings.xml](./attribouter/src/main/res/values/strings.xml) - this file should be copied to ../values-{lang}/strings.xml when translated. (this is an absurdly concise explanation - if this isn't clear, simply sending us translations in a new issue or email is perfectly fine!)
- *Documentation:* Writing guides and explanations of how Attribouter works, how to use it, and how to contribute to it can go a long way to ensuring its usefulness and stability in the future. Whether this involves an update to the README, a tutorial for users and contributors, or adding Javadocs & comments to undocumented parts of the codebase - anything is valid!
